package ndk.sample.androidacoustid;

import android.app.Activity;
import android.content.Intent;
import android.media.MediaCodec;
import android.media.MediaExtractor;
import android.media.MediaFormat;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

import org.acoustid.chromaprint.Chromaprint;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;
import java.util.List;


public class MainActivity extends Activity {

    private void test_chromaprint(File path) throws IOException {
        MediaCodec codec = null;

        int channels = -1;
        int sampleRate = -1;
        long duration = -1;

        MediaExtractor extractor = new MediaExtractor();
        extractor.setDataSource(path.getAbsolutePath());

        int numTracks = extractor.getTrackCount();
        for (int i = 0; i < numTracks; ++i) {
            MediaFormat format = extractor.getTrackFormat(i);
            String mime = format.getString(MediaFormat.KEY_MIME);
            Log.d("MIME", mime);
            if (mime.contains("audio")) {
                channels = format.getInteger(MediaFormat.KEY_CHANNEL_COUNT);
                sampleRate = format.getInteger(MediaFormat.KEY_SAMPLE_RATE);
                duration = format.getLong(MediaFormat.KEY_DURATION) / 1000000;

                Log.d("MIME", "Track: " + i);
                Log.d("MIME", "duration: " + duration);
                Log.d("MIME", "channels: " + channels);
                Log.d("MIME", "sampleRate: " + sampleRate);
                extractor.selectTrack(i);

                codec = MediaCodec.createDecoderByType(mime);
                codec.configure(format, null /* surface */, null /* crypto */, 0 /* flags */);

                break;
            }
        }

        codec.start();

        Chromaprint chromaprint = new Chromaprint();
        chromaprint.start(sampleRate, channels);

        int max_length = 120;
        int remaining = max_length * channels * sampleRate;

        int sampleRead = 0;
        int sampleDecoded = 0;

        long lastSampleTime = 0;
        long totalSamples = 0;

        ByteBuffer containerInputBuffer = ByteBuffer.allocate(8 * 1024);
        for (; sampleRead >= 0 || sampleDecoded >= 0; ) {
            if (sampleRead >= 0) {
                int inputBufferIndex = codec.dequeueInputBuffer(-1);

                containerInputBuffer.clear();
                sampleRead = extractor.readSampleData(containerInputBuffer, 0);

                if (sampleRead >= 0) {
                    lastSampleTime = extractor.getSampleTime();
                }

                codec.getInputBuffer(inputBufferIndex).put(containerInputBuffer);
                codec.queueInputBuffer(inputBufferIndex, 0, sampleRead >= 0 ? sampleRead : 0, sampleRead >= 0 ? lastSampleTime : 0, sampleRead >= 0 ? 0 : MediaCodec.BUFFER_FLAG_END_OF_STREAM);

                extractor.advance();
            }

            if (sampleDecoded >= 0) {
                MediaCodec.BufferInfo bufferInfo = new MediaCodec.BufferInfo();
                int outputBufferIndex = codec.dequeueOutputBuffer(bufferInfo, 10000);

                if (outputBufferIndex >= 0) {
                    ByteBuffer outputBuffer = codec.getOutputBuffer(outputBufferIndex);

                    ShortBuffer sampleBuffer = outputBuffer.asShortBuffer();
                    totalSamples += sampleBuffer.remaining();

                    int length = Math.min(remaining, sampleBuffer.remaining());
                    short[] data = new short[length];
                    sampleBuffer.get(data, 0, data.length);
                    chromaprint.feed(data);

                    codec.releaseOutputBuffer(outputBufferIndex, true);

                    remaining -= length;
                    if (remaining <= 0) {
                        Log.d("FPCALC", "remaining: " + remaining);
                        break;
                    }
                }

                if ((bufferInfo.flags & MediaCodec.BUFFER_FLAG_END_OF_STREAM) != 0) {
                    sampleDecoded = -1;
                }
            }

        }

        Log.d("DECODED", "totalSamples: " + totalSamples);
        Log.d("DECODED", "lastSampleTimeMicros: " + lastSampleTime);

        codec.stop();
        codec.release();
        codec = null;

        extractor.release();
        extractor = null;

        chromaprint.finish();

        String aid = chromaprint.getFingerprint();
        long alen = lastSampleTime / 1000000;
        String apicall = "http://api.acoustid.org/v2/lookup?client=8XaBELgH&meta=recordings&duration=" + duration + "&fingerprint=" + aid;

        Log.d("CHROMAPRINT", aid);
        Log.d("CHROMAPRINT", apicall);

        final Intent intent = new Intent(Intent.ACTION_VIEW).setData(Uri.parse(apicall));
        startActivity(intent);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Get ListView object from xml
        final ListView listView = (ListView) findViewById(R.id.listView);

        File dir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS);

        List<File> music = new ArrayList<File>();
        for (File it : dir.listFiles()) {
            if (it.getPath().endsWith(".m4a") || it.getPath().endsWith(".mp3")) {
                music.add(it);
            }
        }

        ArrayAdapter<File> adapter = new ArrayAdapter<File>(this, android.R.layout.simple_list_item_1, android.R.id.text1, music);
        listView.setAdapter(adapter);

        // ListView Item Click Listener
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> parent, View view,
                                    int position, long id) {

                // ListView Clicked item index
                int itemPosition = position;

                // ListView Clicked item value
                final File itemValue = (File) listView.getItemAtPosition(position);

                // Show Alert
                Toast.makeText(getApplicationContext(), "Please wait. Calculating Chromaprint for " + itemValue.getName(), Toast.LENGTH_LONG)
                        .show();


                new Thread() {
                    @Override
                    public void run() {
                        try {
                            test_chromaprint(itemValue);
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }.start();

            }

        });
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(final MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
